/*  IQNotes - Smarty notes
    Copyright (C) 2001 Peter Vrabel <kybu@kybu.sk>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/
#include "app.h"
#include "aboutBase.h"
#include "notes.h"
#include "defineEntry.h"
#include "entry.h"
#include "chooseEntry.h"
#include "notesFile.h"
#include "qsketch.h"
#include "preferences.h"
#include "rijndael.h"
#include "startUpPasswdBase.h"
#include "fileName.h"
#include "openFile.h"
#include "cfgFile.h"

#include <qdir.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qlineedit.h>
#include <qtooltip.h>
#include <qaction.h>
#include <qxml.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qtoolbutton.h>
#include <qcheckbox.h>
#include <qvaluelist.h>
#include <qlabel.h>
#include <qcstring.h>
#include <qfont.h>
#include <qcombobox.h>
#include <qspinbox.h>
	
#include <qpe/qpemenubar.h>
#include <qpe/qpeapplication.h>
#include <qpe/qpetoolbar.h>
#include <qpe/resource.h>
#include <qpe/filemanager.h>
#include <qpe/fileselector.h>
#include <qpe/global.h>

#include <qtextcodec.h>

App *IQApp;

App::App(QWidget* parent, const char* name, WFlags fl) : QMainWindow(parent, name, fl)
{
    showFullScreen = false;
    setCaption("IQNotes");

    toolbar = new QToolBar(this);
    toolbar->setVerticalStretchable(false);
    toolbar->setHorizontalStretchable(false);
    
    setToolBarsMovable(false);
    multiTB = new QToolButton(toolbar);
    
    // File menu
    filePopupMenu = new QPopupMenu(this);
	menu = new QPEMenuBar(this);

    int fileID;
    fileID= menu->insertItem(tr("ե(&F)"), filePopupMenu);
	
    newID = filePopupMenu->insertItem(Resource::loadPixmap("iqnotes/new"),tr("(&N)"), this, SLOT(newFile()), CTRL+Key_N);
    openID = filePopupMenu->insertItem(Resource::loadPixmap("iqnotes/fileopen"),tr("(&O)"), this, SLOT(openFile()), CTRL+Key_O);
    saveID = filePopupMenu->insertItem(tr("¸(&S)"), this, SLOT(saveFile()), CTRL+Key_S);
    saveAsID = filePopupMenu->insertItem(tr("̾դ¸(&A)"), this, SLOT(saveAsFile()));
    closeID = filePopupMenu->insertItem(Resource::loadPixmap("iqnotes/close"),tr("Ĥ(&C)"), this, SLOT(closeFileMenu()));
    filePopupMenu->insertSeparator();
    removeID = filePopupMenu->insertItem(tr("(&D)"), this, SLOT(removeFile()));
    filePopupMenu->insertSeparator();
    execID = filePopupMenu->insertItem(tr("ޥɼ¹(&F)"), this, SLOT(execCommand()));

    inportPopupMenu = new QPopupMenu(this);
    inportPopupMenu->insertItem(tr("Text(Ȥθ)"), this, SLOT(openFileText()));
    inportPopupMenu->insertItem(tr("WZ(ƤΥĥ꡼о)"), this, SLOT(openFileWZ()));
    inportID = filePopupMenu->insertItem(tr("ݡ(&I)"), inportPopupMenu);

    exportPopupMenu = new QPopupMenu(this);
    exportPopupMenu->insertItem(tr("Text(о)"), this, SLOT(exportText()));
    exportPopupMenu->insertItem(tr("Png(о)"), this, SLOT(exportSketch()));
    exportPopupMenu->insertItem(tr("WZ(ƤΥĥ꡼о)"), this, SLOT(saveTreeWZ()));
    exportID = filePopupMenu->insertItem(tr("ݡ(&E)"), exportPopupMenu);

    filePopupMenu->insertSeparator();
	
    filePopupMenu->insertItem(tr("λ(&Q)"), this, SLOT(goodBye()));

    // Tree menu
    treePopupMenu = new QPopupMenu(this);
    treeID = menu->insertItem(tr("ĥ꡼(&T)"), treePopupMenu);

    searchA = new QAction(tr("(&S)"), Resource::loadPixmap("iqnotes/find"), QString::null, 0, this, 0 );
    connect(searchA, SIGNAL(activated()), this, SLOT(search()));
    searchA->addTo(treePopupMenu);
    multiTB->setIconSet(QIconSet(Resource::loadPixmap("iqnotes/find")));
    connect(multiTB, SIGNAL(clicked()), this, SLOT(search()));

    treePopupMenu->insertSeparator();
    toolbar->addSeparator();

	quickAddA = new QAction(tr("ʰText(&T)"), Resource::loadPixmap("iqnotes/color_line"), QString::null, CTRL+Key_T, this, 0 );
	connect(quickAddA, SIGNAL(activated()), this, SLOT(quickAdd_Text()));
    quickAddA->addTo(toolbar);
    quickAddA->addTo(treePopupMenu);

	quickAddB = new QAction(tr("ʰSketch(&K)"), Resource::loadPixmap("iqnotes/colorize"), QString::null, CTRL+Key_K, this, 0 );
	connect(quickAddB, SIGNAL(activated()), this, SLOT(quickAdd_Sketch()));
    quickAddB->addTo(toolbar);
    quickAddB->addTo(treePopupMenu);

    treePopupMenu->insertSeparator();
    toolbar->addSeparator();

    addBeforeA = new QAction(tr("ɲ(&B)"), Resource::loadPixmap("iqnotes/add_before"), QString::null, CTRL+Key_B, this, 0 );
    connect(addBeforeA, SIGNAL(activated()), this, SLOT(addBefore()));
    addBeforeA->addTo(toolbar);
    addBeforeA->addTo(treePopupMenu);

    addAfterA = new QAction(tr("ɲ(&A)"), Resource::loadPixmap("iqnotes/add_after"), QString::null, CTRL+Key_A, this, 0 );
    connect(addAfterA, SIGNAL(activated()), this, SLOT(addAfter()));
    addAfterA->addTo(toolbar);
    addAfterA->addTo(treePopupMenu);

    addChildA = new QAction(tr("ҤȤɲ(&L)"), Resource::loadPixmap("iqnotes/add_child"), QString::null, CTRL+Key_L, this, 0 );
    connect(addChildA, SIGNAL(activated()), this, SLOT(addChild()));
    addChildA->addTo(toolbar);
    addChildA->addTo(treePopupMenu);

    treePopupMenu->insertSeparator();
    sortID = treePopupMenu->insertItem(tr("(&P)"), this, SLOT(sort()));
    treePopupMenu->insertSeparator();

    expandTreeID = treePopupMenu->insertItem(tr("ƤΥĥ꡼򳫤(&O)"), this, SLOT(expandTree()));
    collapseTreeID = treePopupMenu->insertItem(tr("ƤΥĥ꡼Ĥ(&C)"), this, SLOT(collapseTree()));

    treePopupMenu->insertSeparator();

    taskListPopupMenu = new QPopupMenu(this);
    taskListPopupMenu->insertItem(tr("ƤΥĥ꡼о"), this, SLOT(taskListWholeTree()));
    taskListPopupMenu->insertItem(tr("ȥĥ꡼ʲо"), this, SLOT(taskListCurrent()));

    eventListPopupMenu = new QPopupMenu(this);
    eventListPopupMenu->insertItem(tr("ƤΥĥ꡼о"), this, SLOT(eventListWholeTree()));
    eventListPopupMenu->insertItem(tr("ȥĥ꡼ʲо"), this, SLOT(eventListCurrent()));

    taskListID = treePopupMenu->insertItem(tr("Task(&T)"), taskListPopupMenu);
    eventListID = treePopupMenu->insertItem(tr("Event(&E)"), eventListPopupMenu);

    treePopupMenu->insertSeparator();
    reminderID = treePopupMenu->insertItem("Reminder(&R)", this, SLOT(showReminder()));

    toolbar->addSeparator();

    // Note menu
    notePopupMenu = new QPopupMenu(this);
    noteID = menu->insertItem(tr("Ρ(&N)"), notePopupMenu);

	renameNoteID = notePopupMenu->insertItem(tr("̾ѹ(&R)"), this, SLOT(renameNote()), CTRL+Key_R);

    editA = new QAction(tr("Խ(&E)"), Resource::loadPixmap("iqnotes/edit"), QString::null, CTRL+Key_E, this, 0 );
    editA->setToolTip("Edit");
    connect(editA, SIGNAL(activated()), this, SLOT(editNote()));
    editA->addTo(toolbar);
    editA->addTo(notePopupMenu);

    cutA = new QAction(tr("ڤ(&X)"), Resource::loadPixmap("iqnotes/cut"), QString::null, CTRL+Key_X, this, 0 );
    cutA->setToolTip("Cut note");
    connect(cutA, SIGNAL(activated()), this, SLOT(cutNote()));
    cutA->addTo(toolbar);
    cutA->addTo(notePopupMenu);

    copyNotePopupMenu = new QPopupMenu(this);
    copyNotePopupMenu->insertItem(tr("ȤΤ"), this, SLOT(copyNoteOnlyCurrent()));
    copyNotePopupMenu->insertItem(tr("Ȱʲ"), this, SLOT(copyNoteCurrentAndDown()));
    copyNoteID = notePopupMenu->insertItem(Resource::loadPixmap("iqnotes/copy"),tr("ԡ(&C)"), copyNotePopupMenu);

    pasteNotePopupMenu = new QPopupMenu(this);
    pasteNotePopupMenu->insertItem(tr("Žդ(&B)"), this, SLOT(pasteNoteBefore()));
    pasteNotePopupMenu->insertItem(tr("Žդ(&A)"), this, SLOT(pasteNoteAfter()));
    pasteNotePopupMenu->insertItem(tr("ҤȤŽդ(&L)"), this, SLOT(pasteNoteChild()));
    pasteNoteID = notePopupMenu->insertItem(Resource::loadPixmap("iqnotes/paste"),tr("Žդ(&P)"), pasteNotePopupMenu);

    notePopupMenu->insertSeparator();

    setPictureID = notePopupMenu->insertItem(tr("(&I)"), this, SLOT(setPicture()));

    notePopupMenu->insertSeparator();

    setTaskID = notePopupMenu->insertItem(tr("Task(&T)"), this, SLOT(setTask()));
    setEventID = notePopupMenu->insertItem(tr("Event(&V)"), this, SLOT(setEvent()));
    unsetTaskEventID = notePopupMenu->insertItem(tr("Task/Event(&U)"), this, SLOT(unsetTaskEvent()));

    notePopupMenu->insertSeparator();
    setReminderID = notePopupMenu->insertItem(tr("Reminder(&R)"), this, SLOT(setReminder()));
    unsetReminderID = notePopupMenu->insertItem(tr("Reminder(&N)"), this, SLOT(unsetReminder()));

    // View menu
    viewPopupMenu = new QPopupMenu(this);
    viewID = menu->insertItem(tr("ӥ塼(&V)"), viewPopupMenu);

    toolbar->addSeparator();

    hideNoteA = new QAction(tr("ĥ꡼Τɽ(&T)"), Resource::loadPixmap("iqnotes/hide_note"), QString::null, 0, this, 0 );
    hideNoteA->setToolTip("Hide note");
    connect(hideNoteA, SIGNAL(activated()), this, SLOT(hideNote()));
    hideNoteA->addTo(toolbar);
    hideNoteA->addTo(viewPopupMenu);

    hideTreeA = new QAction(tr("ΡȤΤɽ(&N)"), Resource::loadPixmap("iqnotes/hide_tree"), QString::null, 0, this, 0 );
    hideTreeA->setToolTip("Hide tree");
    connect(hideTreeA, SIGNAL(activated()), this, SLOT(hideTree()));
    hideTreeA->addTo(toolbar);
    hideTreeA->addTo(viewPopupMenu);

    halfViewA = new QAction(tr("ʬɽ(&D)"), Resource::loadPixmap("iqnotes/half_view"), QString::null, 0, this, 0 );
    halfViewA->setToolTip("Half view");
    connect(halfViewA, SIGNAL(activated()), this, SLOT(halfView()));
    halfViewA->addTo(toolbar);
    halfViewA->addTo(viewPopupMenu);
   
    // Options menu
    optionsPopupMenu = new QPopupMenu(this);
    optionsID = menu->insertItem(tr("ץ(&O)"), optionsPopupMenu);

    optionsPopupMenu->insertItem(tr("EntryϿ(&N)"), this, SLOT(defineNewEntry()));
    optionsPopupMenu->insertItem(tr("EntryԽ(&E)"), this, SLOT(changeEntry()));
    optionsPopupMenu->insertItem(tr("Entry(&D)"), this, SLOT(deleteEntry()));

    optionsPopupMenu->insertSeparator();

    optionsPopupMenu->insertItem(tr("ܺ(&P)"), this, SLOT(preferenc()), CTRL+Key_P);

    // Help menu
    helpPopupMenu = new QPopupMenu(this);
    menu->insertItem(tr("Help"), helpPopupMenu);

    helpPopupMenu->insertItem(tr("About"), this, SLOT(about()));

    addToolBar(toolbar);

    IQApp = this;

    //
    notes = new Notes(this, "bla");
    setCentralWidget(notes);

    connect(notes, SIGNAL(emptyNoteTree()), this, SLOT(isEmptyNoteTree()));
    connect(notes, SIGNAL(noEmptyNoteTree()), this, SLOT(isNotEmptyNoteTree()));
    connect(notes, SIGNAL(searchTreeShown()), this, SLOT(searchTreeShown()));
    connect(notes, SIGNAL(searchTreeClosed()), this, SLOT(searchTreeClosed()));
    connect(notes, SIGNAL(taskListShown()), this, SLOT(taskListShown()));
    connect(notes, SIGNAL(taskListClosed()), this, SLOT(taskListClosed()));
    connect(notes, SIGNAL(eventListShown()), this, SLOT(eventListShown()));
    connect(notes, SIGNAL(eventListClosed()), this, SLOT(eventListClosed()));
    connect(notes, SIGNAL(reminderShown()), this, SLOT(reminderShown()));
    connect(notes, SIGNAL(reminderClosed()), this, SLOT(reminderClosed()));

    readConfig();
    changeFont();
    noNoteTree();


}

App::~App()
{}

void App::resizeEvent(QResizeEvent *)
{
    if (showFullScreen && (size() != qApp->desktop()->size()))
        setFullScreen (true);
}

void App::focusInEvent (QFocusEvent *)
{
    if (showFullScreen)
        setFullScreen(true);
}

void App::setFullScreen(bool fullScreen)
{
    static QSize normalsize;

    if (fullScreen)
    {
        if (!normalsize.isValid())
            normalsize = size();

        setFixedSize(qApp->desktop()->size());
        showNormal();
        reparent(0, WStyle_Customize | WStyle_NoBorder, QPoint (0, 0));
        QMainWindow::showFullScreen();
    }
    else
    {
        showNormal();
        reparent(0, 0, QPoint (0, 0));
        resize(normalsize);
        showMaximized();
        normalsize = QSize();
    }
}

bool App::readConfig()
{
    QFile cfgFile(Global::applicationFileName("iqnotes", "iqnotes.cfg"));

    if (!cfgFile.open(IO_ReadOnly))
    {
        if (!cfgFile.open(IO_WriteOnly))
            return false;

        return saveConfig();
    }
    cfgFile.close();

    CfgFileParser handler;
    QXmlInputSource *source;
    QXmlSimpleReader reader;

    connect(&handler, SIGNAL(configLoaded(const NotesConfig &)), this, SLOT(configLoaded(const NotesConfig &)));
    source = new QXmlInputSource(cfgFile);
    reader.setContentHandler(&handler);
    if(!reader.parse(*source))
    {
        qWarning(tr("ConfigFileɤ߹ޤǤ"));
        delete source;

        return false;
    }

    delete source;
    return true;
}

bool App::saveConfig()
{
    QFile cfgFile(Global::applicationFileName("iqnotes", "iqnotes.cfg"));

    if (!cfgFile.open(IO_WriteOnly))
        return false;

    QTextStream cfgText(&cfgFile);
    cfgText << "<?xml version=\"1.0\" encoding=\"Shift_JIS\"?>\n"
            << "<iqnotes>\n"
            << "<config rootnodedecoration=\"" << (config.rootNodeDecoration ? "yes" : "no") << "\" "
            << "showscrollbars=\"" << (config.showScrollBars ? "yes" : "no") << "\" "
            << "wordwrap=\"" << (config.wordWrap ? "yes" : "no") << "\" "
            << "verticalLayout=\"" << (config.verticalLayout ? "yes" : "no") << "\" "
            << "inputDate=\"" << (config.inputDate ? "yes" : "no") << "\" "
            << "font=\"" << (config.font) << "\" "
            << "fontsize=\"" << (config.fontSize) << "\" "
            << "verPos=\"" << (config.verPos) << "\" "
            << "horPos=\"" << (config.horPos) << "\" "
            << "autoSave=\"" << (config.autoSave ? "yes" : "no") << "\" "
            << "autoLoad=\"" << (config.autoLoad ? "yes" : "no") << "\" "
            << "autoLoadFile=\"" << (config.autoLoadFile) << "\" "
            << "dataLocation=\"" << (config.dataLocation) << "\" "
            << "/>\n"
            << "</iqnotes>\n";

    cfgFile.close();

    return true;
}

void App::configLoaded(const NotesConfig &c)
{
    config = c;
}

void App::closeEvent(QCloseEvent *ce)
{
    if (!closeFile())
        ce->ignore();
    else
        ce->accept();

    return;
}

void App::keyPressEvent(QKeyEvent *ke)
{
     Notes::EViewType viewType = notes->getViewType();
    if (ke->key() == Qt::Key_F10 && !noFileOpened) {

        if (viewType == Notes::HALF_VIEW)
            hideTree();
        else if (viewType == Notes::HIDE_TREE)
            hideNote();
        else if (viewType == Notes::HIDE_NOTE)
            halfView();

        ke->accept();
        return;
    }

	if (viewType == Notes::HALF_VIEW){
	    if (ke->key() == Qt::CTRL+Key_U ) {
	//		QMessageBox::warning(this, tr("ٹ"), "test", 0, 0, 0);
			if(config.verticalLayout){
				if(config.horPos > 1)
					config.horPos = config.horPos - 1;
				notes->setHorPos(config.horPos);
			}else{
				if(config.verPos < 9)
					config.verPos = config.verPos + 1;
				notes->setVerPos(config.verPos);
			}
            hideTree();
	        halfView();
		}
	    if (ke->key() == Qt::CTRL+Key_D ) {
			if(config.verticalLayout){
				if(config.horPos < 9)
					config.horPos = config.horPos + 1;
				notes->setHorPos(config.horPos);
			}else{
				if(config.verPos > 1)
					config.verPos = config.verPos - 1;
				notes->setVerPos(config.verPos);
			}
	        hideTree();
	        halfView();
		}
	    if (ke->key() == Qt::CTRL+Key_V ) {
	    	config.verticalLayout = !(config.verticalLayout);
	    	notes->setVerticalLayout(config.verticalLayout);
	        hideTree();
	        halfView();
		}
	}

    ke->ignore();
}

void App::createNotes()
{
    entriesList = new Entries();

    preferences = Preferences();

    notes->createNotes();

    notes->setWordWrap(config.wordWrap);
    notes->setRootNodeDecoration(config.rootNodeDecoration);
    notes->setShowScrollBars(config.showScrollBars);
    notes->setVerticalLayout(config.verticalLayout);
    notes->setInputDate(config.inputDate);
    notes->setVerPos(config.verPos);
    notes->setHorPos(config.horPos);

    isEmptyNoteTree();
    halfView();

    noFileOpened = false;
}

void App::destroyNotes()
{
    notes->destroyNotes();

    if (entriesList)
        delete entriesList;
    entriesList = 0;

    noNoteTree();

    currentFile = "";
    noFileOpened = true;
}

void App::setBackupLocation(bool save, const QString &location)
{
    preferences.saveBackupFile = save;
    preferences.backupLocation = location;
}

void App::setNotesShowReminderAtStartUp(bool showReminder_)
{
    preferences.showReminder = showReminder_;
}

void App::setSecurityPasswd(const QString &passwd)
{
    preferences.passwdHex = passwd;
}

void App::changeFont()
{
    if (config.font == "" || config.fontSize == 0)
        return;

    QFont f(config.font, config.fontSize);

    qApp->setFont(f, true);
}

const QString App::quoteXML(const QString &s)
{
    QString r;

    for (uint i = 0; i < s.length(); i++)
    {
        if (s[i] == '"')
            r += "&quot;";
        else if (s[i] == '>')
            r += "&gt;";
        else if (s[i] == '<')
            r += "&lt;";
        else if (s[i] == '&')
            r += "&amp;";
        else
            r += s[i];
    }

    return r;
}

void App::addEntry(Entry *e)
{
    entriesList->addEntry(e);
}

void App::deleteEntry(const QString &entryName)
{
    Entry *entry;
    uint i = 0;

    if (notes->isEntryUsed(entryName))
        emit canDeleteEntry(entryName, false);
    else
        emit canDeleteEntry(entryName, true);

    for (entry = entriesList->first(); entry; entry = entriesList->next())
    {
        if (entry->getName() == entryName)
            break;

        i++;
    }

    entriesList->remove
    (i);
}

void App::noNoteTree()
{
    noFileOpened = true;
    isEmptyNoteTree();

    menu->setItemEnabled(saveID, false);
    menu->setItemEnabled(saveAsID, false);
    menu->setItemEnabled(exportID, false);
    menu->setItemEnabled(closeID, false);
    menu->setItemEnabled(treeID, false);
    menu->setItemEnabled(noteID, false);
    menu->setItemEnabled(viewID, false);
    menu->setItemEnabled(optionsID, false);

    quickAddA->setEnabled(false);
    quickAddB->setEnabled(false);
    addBeforeA->setEnabled(false);
    addAfterA->setEnabled(false);
    addChildA->setEnabled(false);

    hideTreeA->setEnabled(false);
    hideNoteA->setEnabled(false);
    halfViewA->setEnabled(false);
}

void App::isEmptyNoteTree()
{
    multiTB->setEnabled(false);
    editA->setEnabled(false);
    cutA->setEnabled(false);
    menu->setItemEnabled(copyNoteID, false);
    if (notes->isClipboardEmpty())
        menu->setItemEnabled(pasteNoteID, false);
    else
        menu->setItemEnabled(pasteNoteID, true);
    
    searchA->setEnabled(false);

    menu->setItemEnabled(sortID, false);
    menu->setItemEnabled(expandTreeID, false);
    menu->setItemEnabled(collapseTreeID, false);
    menu->setItemEnabled(taskListID, false);
    menu->setItemEnabled(eventListID, false);
    menu->setItemEnabled(reminderID, false);

	menu->setItemEnabled(renameNoteID, false);
    menu->setItemEnabled(setPictureID, false);
    menu->setItemEnabled(setTaskID, false);
    menu->setItemEnabled(setEventID, false);
    menu->setItemEnabled(unsetTaskEventID, false);
    menu->setItemEnabled(setReminderID, false);
    menu->setItemEnabled(unsetReminderID, false);
    menu->setItemEnabled(exportID, false);

    quickAddA->setEnabled(true);
    quickAddB->setEnabled(true);
    addBeforeA->setEnabled(true);
    addAfterA->setEnabled(true);
    addChildA->setEnabled(true);

    menu->setItemEnabled(saveID, true);
    menu->setItemEnabled(saveAsID, true);
    menu->setItemEnabled(closeID, true);
    menu->setItemEnabled(treeID, true);
    menu->setItemEnabled(noteID, true);
    menu->setItemEnabled(viewID, true);
    menu->setItemEnabled(optionsID, true);
}

void App::isNotEmptyNoteTree()
{
    multiTB->setEnabled(true);
    editA->setEnabled(true);
    cutA->setEnabled(true);
    menu->setItemEnabled(copyNoteID, true);
    menu->setItemEnabled(pasteNoteID, true);

    searchA->setEnabled(true);

    menu->setItemEnabled(sortID, true);
    menu->setItemEnabled(expandTreeID, true);
    menu->setItemEnabled(collapseTreeID, true);
    menu->setItemEnabled(taskListID, true);
    menu->setItemEnabled(eventListID, true);
    menu->setItemEnabled(reminderID, true);
    menu->setItemEnabled(exportID, true);

	menu->setItemEnabled(renameNoteID, true);
    menu->setItemEnabled(setPictureID, true);
    menu->setItemEnabled(setTaskID, true);
    menu->setItemEnabled(setEventID, true);
    menu->setItemEnabled(unsetTaskEventID, true);
    menu->setItemEnabled(setReminderID, true);
    menu->setItemEnabled(unsetReminderID, true);

    quickAddA->setEnabled(true);
    quickAddB->setEnabled(true);
    addBeforeA->setEnabled(true);
    addAfterA->setEnabled(true);
    addChildA->setEnabled(true);

    menu->setItemEnabled(saveID, true);
    menu->setItemEnabled(saveAsID, true);
    menu->setItemEnabled(closeID, true);
    menu->setItemEnabled(treeID, true);
    menu->setItemEnabled(noteID, true);
    menu->setItemEnabled(viewID, true);
    menu->setItemEnabled(optionsID, true);

}

void App::taskListShown()
{
    treePopupMenu->removeItem(taskListID);
    taskListID = treePopupMenu->insertItem(Resource::loadPixmap("iqnotes/close_search_tree"), tr("TaskĤ(&C)"), notes, SLOT(taskListClose()), 0, -1, 11);
    multiTB->disconnect();
    multiTB->setIconSet(Resource::loadPixmap("iqnotes/close_search_tree"));
    connect(multiTB, SIGNAL(clicked()), notes, SLOT(taskListClose()));

    searchA->setEnabled(false);

    quickAddA->setEnabled(false);
    quickAddB->setEnabled(false);
    addBeforeA->setEnabled(false);
    addAfterA->setEnabled(false);
    addChildA->setEnabled(false);

    menuBar()->setItemEnabled(expandTreeID, false);
    menu->setItemEnabled(collapseTreeID, false);

    menu->setItemEnabled(eventListID, false);
    menu->setItemEnabled(reminderID, false);
    menu->setItemEnabled(exportID, false);

    menu->setItemEnabled(setEventID, false);
    menu->setItemEnabled(unsetTaskEventID, false);

    cutA->setEnabled(false);
    menu->setItemEnabled(copyNoteID, false);
    menu->setItemEnabled(pasteNoteID, false);
}

void App::taskListClosed()
{
    treePopupMenu->removeItem(taskListID);
    taskListID = taskListID = treePopupMenu->insertItem(tr("Task(&T)"), taskListPopupMenu, -1, 11);
    multiTB->disconnect();
    multiTB->setIconSet(Resource::loadPixmap("iqnotes/find"));
    connect(multiTB, SIGNAL(clicked()), this, SLOT(search()));

    searchA->setEnabled(true);

    quickAddA->setEnabled(true);
    quickAddB->setEnabled(true);
    addBeforeA->setEnabled(true);
    addAfterA->setEnabled(true);
    addChildA->setEnabled(true);

    menu->setItemEnabled(expandTreeID, true);
    menu->setItemEnabled(collapseTreeID, true);

    menu->setItemEnabled(eventListID, true);
    menu->setItemEnabled(reminderID, true);
    menu->setItemEnabled(exportID, true);

    menu->setItemEnabled(setEventID, true);
    menu->setItemEnabled(unsetTaskEventID, true);

    cutA->setEnabled(true);
    menu->setItemEnabled(copyNoteID, true);
    menu->setItemEnabled(pasteNoteID, true);
}

void App::eventListShown()
{
    treePopupMenu->removeItem(eventListID);
    eventListID = treePopupMenu->insertItem(Resource::loadPixmap("iqnotes/close_search_tree"), tr("EventĤ(&C)"), notes, SLOT(eventListClose()), 0, -1, 12);
    multiTB->disconnect();
    multiTB->setIconSet(Resource::loadPixmap("iqnotes/close_search_tree"));
    connect(multiTB, SIGNAL(clicked()), notes, SLOT(eventListClose()));

    searchA->setEnabled(false);

    quickAddA->setEnabled(false);
    quickAddB->setEnabled(false);
    addBeforeA->setEnabled(false);
    addAfterA->setEnabled(false);
    addChildA->setEnabled(false);

    menu->setItemEnabled(expandTreeID, false);
    menu->setItemEnabled(collapseTreeID, false);
    menu->setItemEnabled(reminderID, false);
    menu->setItemEnabled(exportID, false);

    menu->setItemEnabled(taskListID, false);

    menu->setItemEnabled(setTaskID, false);
    menu->setItemEnabled(unsetTaskEventID, false);

    cutA->setEnabled(false);
    menu->setItemEnabled(copyNoteID, false);
    menu->setItemEnabled(pasteNoteID, false);
}

void App::eventListClosed()
{
    treePopupMenu->removeItem(eventListID);
    eventListID = treePopupMenu->insertItem(tr("Event(&E)"), eventListPopupMenu, -1, 12);
    multiTB->disconnect();
    multiTB->setIconSet(Resource::loadPixmap("iqnotes/find"));
    connect(multiTB, SIGNAL(clicked()), this, SLOT(search()));

    searchA->setEnabled(true);

    quickAddA->setEnabled(true);
    quickAddB->setEnabled(true);
    addBeforeA->setEnabled(true);
    addAfterA->setEnabled(true);
    addChildA->setEnabled(true);

    menu->setItemEnabled(expandTreeID, true);
    menu->setItemEnabled(collapseTreeID, true);

    menu->setItemEnabled(taskListID, true);
    menu->setItemEnabled(reminderID, true);
    menu->setItemEnabled(exportID, true);

    menu->setItemEnabled(setTaskID, true);
    menu->setItemEnabled(unsetTaskEventID, true);

    cutA->setEnabled(true);
    menu->setItemEnabled(copyNoteID, true);
    menu->setItemEnabled(pasteNoteID, true);
}

void App::searchTreeShown()
{
    searchA->setIconSet(Resource::loadPixmap("iqnotes/close_search_tree"));
    searchA->setText(tr("̤Ĥ(&C)"));
    multiTB->setIconSet(Resource::loadPixmap("iqnotes/close_search_tree"));

    quickAddA->setEnabled(false);
    quickAddB->setEnabled(false);
    addBeforeA->setEnabled(false);
    addAfterA->setEnabled(false);
    addChildA->setEnabled(false);

    menu->setItemEnabled(taskListID, false);
    menu->setItemEnabled(eventListID, false);
    menu->setItemEnabled(reminderID, false);
    menu->setItemEnabled(exportID, false);

    menu->setItemEnabled(sortID, false);

    cutA->setEnabled(false);
    menu->setItemEnabled(copyNoteID, false);
    menu->setItemEnabled(pasteNoteID, false);
}

void App::searchTreeClosed()
{
    searchA->setIconSet(Resource::loadPixmap("iqnotes/find"));
    searchA->setText(tr("(&S)"));
    multiTB->setIconSet(Resource::loadPixmap("iqnotes/find"));

    quickAddA->setEnabled(true);
    quickAddB->setEnabled(true);
    addBeforeA->setEnabled(true);
    addAfterA->setEnabled(true);
    addChildA->setEnabled(true);

    menu->setItemEnabled(taskListID, true);
    menu->setItemEnabled(eventListID, true);
    menu->setItemEnabled(reminderID, true);
    menu->setItemEnabled(exportID, true);

    menu->setItemEnabled(sortID, true);

    cutA->setEnabled(true);
    menu->setItemEnabled(copyNoteID, true);
    menu->setItemEnabled(pasteNoteID, true);
}

void App::reminderShown()
{
    treePopupMenu->removeItem(reminderID);
    reminderID = treePopupMenu->insertItem(Resource::loadPixmap("iqnotes/close_search_tree"), tr("ReminderĤ(&C)"), notes, SLOT(reminderClose()));
    multiTB->disconnect();
    multiTB->setIconSet(Resource::loadPixmap("iqnotes/close_search_tree"));
    connect(multiTB, SIGNAL(clicked()), notes, SLOT(reminderClose()));

    searchA->setEnabled(false);

    quickAddA->setEnabled(false);
    quickAddB->setEnabled(false);
    addBeforeA->setEnabled(false);
    addAfterA->setEnabled(false);
    addChildA->setEnabled(false);

    menu->setItemEnabled(expandTreeID, false);
    menu->setItemEnabled(collapseTreeID, false);
    menu->setItemEnabled(exportID, false);

    menu->setItemEnabled(taskListID, false);
    menu->setItemEnabled(eventListID, false);

    menu->setItemEnabled(unsetReminderID, false);

    cutA->setEnabled(false);
    menu->setItemEnabled(copyNoteID, false);
    menu->setItemEnabled(pasteNoteID, false);
}

void App::reminderClosed()
{
    treePopupMenu->removeItem(reminderID);
    reminderID = treePopupMenu->insertItem(tr("Reminder"), this, SLOT(showReminder()));
    multiTB->disconnect();
    multiTB->setIconSet(Resource::loadPixmap("iqnotes/find"));
    connect(multiTB, SIGNAL(clicked()), this, SLOT(search()));

    searchA->setEnabled(true);

    quickAddA->setEnabled(true);
    quickAddB->setEnabled(true);
    addBeforeA->setEnabled(true);
    addAfterA->setEnabled(true);
    addChildA->setEnabled(true);

    menu->setItemEnabled(expandTreeID, true);
    menu->setItemEnabled(collapseTreeID, true);
    menu->setItemEnabled(exportID, true);

    menu->setItemEnabled(taskListID, true);
    menu->setItemEnabled(eventListID, true);

    menu->setItemEnabled(unsetReminderID, true);

    cutA->setEnabled(true);
    menu->setItemEnabled(copyNoteID, true);
    menu->setItemEnabled(pasteNoteID, true);
}

void App::openTree(int index, const QString &file)
{
    // no file selected
    if (index == -1)
        return;

    // new data file
    if (index == -2)
    {
        newFile();

        return;
    }

    createNotes();
    if (!openTree(file))
    {
        destroyNotes();

        return;
    }
    
    currentFile = file;
    setModified(false);
    
    foreignDeletedNode = !notes->isClipboardEmpty();
    
    if (preferences.showReminder)
        notes->showReminder(false);
}

bool App::openTree(const QString &file)
{
    NotesFileParser handler;

    connect(&handler, SIGNAL(backupLoaded(bool, const QString &)), this, SLOT(setBackupLocation(bool, const QString &)));
    connect(&handler, SIGNAL(notesShowReminderAtStartUpLoaded(bool)), this, SLOT(setNotesShowReminderAtStartUp(bool)));
//    connect(&handler, SIGNAL(notesWordWrapLoaded(bool)), this, SLOT(setNotesWordWrap(bool)));
//    connect(&handler, SIGNAL(notesRootNodeDecorationLoaded(bool)), this, SLOT(setNotesRootNodeDecoration(bool)));
//    connect(&handler, SIGNAL(notesShowScrollBarsLoaded(bool)), this, SLOT(setNotesShowScrollBars(bool)));
    connect(&handler, SIGNAL(entryLoaded(Entry *)), this, SLOT(addEntry(Entry *)));
    connect(&handler, SIGNAL(noteLoaded(const QString &, QList<QString> *, const QString &, const QString &, int, QDateTime, const QString &, QDateTime)), notes, SLOT(addNote(const QString &, QList<QString> *, const QString &, const QString &, int, QDateTime, const QString &, QDateTime)));
    connect(&handler, SIGNAL(securityPasswdLoaded(const QString &)), this, SLOT(setSecurityPasswd(const QString &)));
    connect(&handler, SIGNAL(noteLoaded(const QString &, Strokes *, const QString &, const QString &, int, QDateTime, const QString &, QDateTime)),
            notes, SLOT(addNote(const QString &, Strokes *, const QString &, const QString &, int, QDateTime, const QString &, QDateTime)));
    connect(&handler, SIGNAL(noNoteChild()), notes, SLOT(noNoteChild()));

	QString tmp = "";
	if(config.dataLocation == "")
		tmp = Global::applicationFileName("iqnotes", "");
	else
		tmp = config.dataLocation;

    QFile xmlFile(tmp + file + ".xml"),xmlRijndaelFile(tmp + file + ".rijn");
    QXmlInputSource *source;

    // Init xml file
    if (!xmlFile.exists() && !xmlRijndaelFile.exists())
    {
        xmlFile.open(IO_WriteOnly);
        QTextStream ts(&xmlFile);

        ts << "<?xml version=\"1.0\" encoding=\"EUC-JP\"?>\n<iqnotes>\n<entries>\n"

        << "<entry name=\"" << tr("ޥɼ¹") << "\">\n"
        << "<property name=\"" << tr("ץꥱ̾") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("ưѥ᥿") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("") << "\" type=\"multiLine\"/>\n"
        << "</entry>\n"

        << "<entry name=\"" << tr("Ҵط") << "\">\n"
        << "<property name=\"" << tr("̾") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("") << "\" type=\"multiLine\"/>\n"
        << "<property name=\"" << tr("ֹ") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("᡼륢ɥ쥹") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("") << "\" type=\"multiLine\"/>\n"
        << "</entry>\n"

        << "<entry name=\"" << tr("Ŀ") << "\">\n"
        << "<property name=\"" << tr("̾") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("˥å͡") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("Ż轻") << "\" type=\"multiLine\"/>\n"
        << "<property name=\"" << tr("Żֹ") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("Ż᡼륢ɥ쥹") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("𽻽") << "\" type=\"multiLine\"/>\n"
        << "<property name=\"" << tr("ֹ") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("᡼륢ɥ쥹") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("") << "\" type=\"multiLine\"/>\n"
        << "</entry>\n"

        << "<entry name=\"" << tr("桼") << "\">\n"
        << "<property name=\"" << tr("ۥ̾") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("桼̾") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("ѥ") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("") << "\" type=\"mutliLine\"/>\n"
        << "</entry>\n"

        << "<entry name=\"" << tr("쥸åȥ") << "\">\n"
        << "<property name=\"" << tr("ɥ") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("ֹ") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("λ") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("žֹ") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("") << "\" type=\"mutliLine\"/>\n"
        << "</entry>\n"

        << "<entry name=\"" << tr("URL") << "\">\n"
        << "<property name=\"" << tr("ȥ") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("URL") << "\" type=\"oneLine\"/>\n"
        << "<property name=\"" << tr("") << "\" type=\"multiLine\"/>\n"
        << "</entry>\n"

        << "</entries>\n"
        << "<notes>\n</notes>\n</iqnotes>\n";

        xmlFile.close();
        source = new QXmlInputSource(xmlFile);
    }
    else if (xmlRijndaelFile.exists())
    {
        StartUpPasswdBase passwdD(this, 0, true);
        passwdD.adjustSize();
        if (!passwdD.exec())
        {
            return false;
        }
        
        preferences.setPasswd(passwdD.Passwd->text());

        xmlRijndaelFile.open(IO_ReadOnly);
        uint fileSize = xmlRijndaelFile.size();
        unsigned char *rijnData = (unsigned char *) malloc(fileSize),
            *rijnDataDecrypted = (unsigned char *) malloc(fileSize);
        Rijndael rijndael;

        xmlRijndaelFile.readBlock((char *)rijnData, fileSize);
        rijndael.init(Rijndael::CBC, Rijndael::Decrypt, preferences.passwd16Bin, Rijndael::Key16Bytes);
        rijndael.padDecrypt(rijnData, fileSize, rijnDataDecrypted);

		// yeah, stupid
        if (memcmp((void *)"<iqnotes>", (void *)rijnDataDecrypted, 9) != 0 && memcmp((void *)"<?xml version", (void *)rijnDataDecrypted, 13) != 0)
        {
            free(rijnData);
            free(rijnDataDecrypted);

            QMessageBox::critical(this, tr("ѥɤθ"), tr("ѥɤϤƤ"));

            return false;
        }

        source = new QXmlInputSource();
        source->setData(QString::fromUtf8((char *)rijnDataDecrypted));

        free(rijnData);
        free(rijnDataDecrypted);
    }
    else
    {
        source = new QXmlInputSource(xmlFile);
    }

    QXmlSimpleReader reader;
    reader.setContentHandler(&handler);
    reader.setErrorHandler(new NotesFileParserError);
    if(!reader.parse(*source))
        qWarning(tr("parse error"));

    delete source;

    setCaption("IQNotes :: " + file);
     
    return true;
}

bool App::saveTree()
{
    // backup dir checking (see also preferences.cpp)
    if (preferences.saveBackupFile)
    {
        QDir backupDir(preferences.backupLocation);
        
        if (!backupDir.exists())
        {
            QMessageBox::critical(this, "̵ʥǥ쥯ȥ", "backup¸ǥ쥯ȥϤƤ", 0, 0);
            return false;
        }
    }
    
    QString tmp = "";
	if(config.dataLocation == ""){
		tmp = Global::applicationFileName("iqnotes","");
	}else{
		tmp = config.dataLocation;
	}
    QFile saveF(tmp + currentFile + ".xml");
    QFile rijnF(tmp + currentFile + ".rijn");

    QString saveS;
    QTextStream saveData(saveS, IO_WriteOnly);

    saveData << "<?xml version=\"1.0\" encoding=\"Shift_JIS\"?>\n<iqnotes>\n"
    << "<config>\n"
    << "<backup save=\"" << (preferences.saveBackupFile ? "yes" : "no") << "\" location=\"" << preferences.backupLocation << "\"/>\n"
             << "<notes showreminder=\"" << (preferences.showReminder ? "yes" : "no") << "\" />\n"
    << "<security passwd=\"" << preferences.passwdHex << "\"/>\n"
    << "</config>\n"

    << "<entries>\n";

    for (Entry *e = entriesList->first(); e; e = entriesList->next())
    {
        saveData << "<entry name=\"" << quoteXML(e->getName()) << "\" defaultpic=\"" << quoteXML(e->getDefaultPic()) << "\">\n";

        for (PropertyStruct *ps = e->first(); ps; ps = e->next())
        {
            saveData << "<property name=\"" << quoteXML(ps->getName()) << "\" "
            << "type=\"" << (ps->getType() == PropertyStruct::ONE_LINE ? "oneLine" : "multiLine") << "\"/>\n";
        }

        saveData << "</entry>\n";
    }

    saveData << "</entries>\n";

    notes->saveTree(&saveData);

    saveData << "</iqnotes>\n";

//    QCString dataUTF8 = saveS.utf8();
	QTextCodec *coder=QTextCodec::codecForName("ShiftJIS");
	QString dataUTF8 = saveS;

    if (!preferences.passwdHex.length())
    {
        saveF.open(IO_WriteOnly);

        QTextStream saveData1(&saveF);
		saveData1.setCodec(coder);
        saveData1 << dataUTF8;

        if (preferences.saveBackupFile)
        {
            QFile backupSaveF(preferences.backupLocation + "/" + currentFile + ".xml");
            QTextStream saveData2(&backupSaveF);
            backupSaveF.open(IO_WriteOnly);
            saveData2 << dataUTF8;
        }

        if (rijnF.exists())
            rijnF.remove();
    }
    else
    {
    	QCString dataUTF8_ = dataUTF8.utf8();

        Rijndael rijndael;
        unsigned char *rijnEncrypt = (unsigned char*)malloc(dataUTF8_.length() + 16);

        rijnF.open(IO_WriteOnly);

        rijndael.init(Rijndael::CBC, Rijndael::Encrypt, preferences.passwd16Bin, Rijndael::Key16Bytes);
        int len = rijndael.padEncrypt((unsigned char*)(const char*)dataUTF8_, dataUTF8_.length(), rijnEncrypt);

        rijnF.writeBlock((const char *)rijnEncrypt, len);
        if (preferences.saveBackupFile)
        {
            QFile rijnBackupF(preferences.backupLocation + "/" + currentFile + ".rijn");
            rijnBackupF.open(IO_WriteOnly);
            rijnBackupF.writeBlock((const char *)rijnEncrypt, len);
        }

        free(rijnEncrypt);

        if (saveF.exists())
            saveF.remove();
    }

    return true;
}

bool App::saveTreeWZ()
{
    QString tmp = "";
	if(config.dataLocation == ""){
		tmp = Global::applicationFileName("iqnotes","");
	}else{
		tmp = config.dataLocation;
	}

	QDateTime dt = QDateTime::currentDateTime();
	QString tmp2 = "";
	tmp2.sprintf("%04d%02d%02d_%02d%02d", dt.date().year(),dt.date().month(),dt.date().day(),dt.time().hour(),dt.time().minute());

	QTextCodec *coder=QTextCodec::codecForName("ShiftJIS");
	QFile saveF(tmp + tmp2 + ".txt");
	saveF.open(IO_WriteOnly);
	QTextStream saveData(&saveF);
	saveData.setCodec(coder);

    notes->saveTreeWZ(&saveData);

	saveF.close();

    QMessageBox::information(this, tr("WZ"), tr("ΡȤWZ\n" + tmp + tmp2 + ".txt\n˽Ϥޤ"));


    return true;
}

void App::setModified(bool m)
{
    if (m)
    {
        modified = true;

        menu->setItemEnabled(saveID, true);
        menu->setItemEnabled(saveAsID, true);
        
        if (currentFile.length())
            setCaption("IQNotes :: " + currentFile + "*");
        else
            setCaption("IQNotes :: (untitled)*");
    }
    else
    {
        modified = false;

        menu->setItemEnabled(saveID, false);
        
        if (currentFile.length())
            setCaption("IQNotes :: " + currentFile);
        else
            setCaption("IQNotes :: (untitled)");
    }
}

// {{{ Methods called from menu

// {{{ File menu

void App::newFile()
{
    if (!closeFile())
        return;

    createNotes();
    if (!openTree("init_file"))
    {
        destroyNotes();

        return;
    }

    currentFile = "";
    setModified(false);
    setCaption("IQNotes :: (untitled)");
    
    foreignDeletedNode = !notes->isClipboardEmpty();
}

void App::openFile()
{
    openFile(false);
}

void App::openFile(bool showNewFile)
{
    if (!closeFile())
        return;

	if((config.autoLoad)&&(showNewFile)){
		newFile();
	    if (!openTree(config.autoLoadFile))
	    {
	        destroyNotes();
	        return;
	    }
	    currentFile = config.autoLoadFile;
	}else{

	    OpenFile of(1,config.dataLocation,showNewFile, this, 0, true);
	    connect(&of, SIGNAL(fileSelected(int, const QString &)), this, SLOT(openTree(int, const QString &)));

	    of.showMaximized();

	    if (!of.exec())
	       return;
	}

}

bool App::saveFile()
{
    if (noFileOpened)
        return true;

    if (currentFile == "" && !noFileOpened)
    {
        FileName fn(config.dataLocation,this, 0, true);

        if (!fn.exec())
            return false;

        currentFile = fn.FileNameI->text();
    }

    if (!saveTree())
        return false;

    setCaption("IQNotes :: " + currentFile);
    setModified(false);

    return true;
}


bool App::saveAsFile()
{

    if (noFileOpened)
        return true;

    FileName fn(config.dataLocation,this, 0, true);

    if (!fn.exec())
        return false;

    currentFile = fn.FileNameI->text();

    if (!saveTree())
        return false;

    setCaption("IQNotes :: " + currentFile);
    setModified(false);

    return true;
}

void App::closeFileMenu()
{
    closeFile();
}

bool App::closeFile()
{
	
    if (noFileOpened)
        return true;

    if (modified)
    {
    	if(config.autoSave){
            if (!saveFile())
                return false;
    	}else{
	        int r = QMessageBox::warning(this, tr("ٹ"), tr("ǡ¸Ƥޤ\n¸ޤ"), QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Cancel | QMessageBox::Escape);

	        if (r == QMessageBox::Yes)
	        {
	            if (!saveFile())
	                return false;
	        }
	        
	        if ((r == QMessageBox::Cancel) || (r == 0))
	            return false;
	    }
    }

    destroyNotes();

    currentFile = "";
    noFileOpened = true;

    setCaption("IQNotes");

    return true;
}

void App::goodBye()
{
    if (!closeFile())
        return;

    close();
}

// }}}

// {{{ Tree menu

void App::search()
{
    notes->search();
}

bool App::foreignNodeWarning()
{
    if (!foreignDeletedNode)
        return true;

    int r = QMessageBox::warning(this, tr("ΡȤɲ"), tr("̤ΥΡȥե뤫ǡɲä褦ȤƤޤ\nŪ˴ʹ԰٤Ǥ\nܤHelpեΡTree/Add beforeפ򻲾ȤƤ\n³ޤ"), QMessageBox::Yes | QMessageBox::Default, QMessageBox::No | QMessageBox::Escape);
    
    if (r == QMessageBox::Yes)
        return true;
    else
        return false;
}

void App::quickAdd_Text()
{
	if (notes->quickAdd_Text())
		setModified(true);
}

void App::quickAdd_Sketch()
{
	if (notes->quickAdd_Sketch())
		setModified(true);
}

void App::addBefore()
{
    if (notes->addBefore())
        setModified(true);
}

void App::addAfter()
{
    if (notes->addAfter())
        setModified(true);
}

void App::addChild()
{
    if (notes->addChild())
        setModified(true);
}

void App::sort()
{
    notes->sort();
    setModified(true);
}

void App::expandTree()
{
    notes->expandTree();
}

void App::collapseTree()
{
    notes->collapseTree();
}

void App::taskListWholeTree()
{
    notes->taskListWholeTree();
}

void App::taskListCurrent()
{
    notes->taskListCurrent();
}

void App::eventListWholeTree()
{
    notes->eventListWholeTree();
}

void App::eventListCurrent()
{
    notes->eventListCurrent();
}

void App::showReminder()
{
    notes->showReminder();
}

// }}}

// {{{ Note menu

void App::cutNote()
{
    if (notes->deleteNote())
        setModified(true);
}

void App::copyNoteOnlyCurrent()
{
    notes->copyNoteOnlyCurrent();
}

void App::copyNoteCurrentAndDown()
{
    notes->copyNoteCurrentAndDown();
}

void App::pasteNoteBefore()
{
    if (notes->pasteNoteBefore())
        setModified(true);
}

void App::pasteNoteAfter()
{
    if (notes->pasteNoteAfter())
        setModified(true);
}

void App::pasteNoteChild()
{
    if (notes->pasteNoteChild())
        setModified(true);
}

void App::renameNote()
{
	if (notes->renameNote())
		setModified(true);
}

void App::editNote()
{
    if (notes->editNote())
        setModified(true);
}

void App::setPicture()
{
    if (notes->setPicture())
        setModified(true);
}

void App::setTask()
{
    if (notes->setTask())
        setModified(true);
}

void App::setEvent()
{
    if (notes->setEvent())
        setModified(true);
}

void App::unsetTaskEvent()
{
    if (notes->unsetTaskEvent())
        setModified(true);
}

void App::setReminder()
{
    if (notes->setReminder())
        setModified(true);
}

void App::unsetReminder()
{
    if (notes->unsetReminder())
        setModified(true);
}

// }}}

// {{{ View menu

void App::hideTree()
{
    notes->fullNoteView();

    hideTreeA->setEnabled(false);
    hideNoteA->setEnabled(true);
    halfViewA->setEnabled(true);
}

void App::hideNote()
{
    notes->fullTreeView();

    hideTreeA->setEnabled(true);
    hideNoteA->setEnabled(false);
    halfViewA->setEnabled(true);
}

void App::halfView()
{
    notes->halfView();

    hideTreeA->setEnabled(true);
    hideNoteA->setEnabled(true);
    halfViewA->setEnabled(false);
}

void App::toggleToolBar()
{
    if (toolbar->isVisible())
        toolbar->hide();
    else
        toolbar->show();
}

void App::toggleFullScreen()
{
    showFullScreen = !showFullScreen;
    setFullScreen (showFullScreen);
}       

// }}}

// {{{ Options menu

void App::defineNewEntry()
{
    DefineEntry de(this, 0, true);
    de.showMaximized();
    if (de.exec())
        setModified(true);
}

void App::changeEntry()
{
    ChooseEntry ce(this, 0, true);
    Entry *entry;

    ce.fillList(entriesList);
    if (!ce.exec())
        return;

    if (ce.currentEntry() == -1)
        return;

    entry = entriesList->at(ce.currentEntry());

    DefineEntry de(this, 0, true);

    if (notes->isEntryUsed(entry->getName()))
    {
        QMessageBox::warning(this, tr("ٹ"), tr("EntryϥΡȤǻȤƤޤ\nEntryˤ줿ѹϤΥפΤ٤ƤΥΡȤ˱ƶޤ"), 0, 0, 0);

        de.setIsUsed(true);
    }

    for (PropertyStruct *ps = entry->first(); ps; ps = entry->next())
    {
        de.addProperty(new PropertyBoxItem(ps));
    }

    de.EntryName->setText(entry->getName());
    de.setEditEntry(entry);
    de.showMaximized();
	
    int r = de.exec();
    if (r && de.getIsUsed())
    {
        notes->deleteEntryProperties(entry->getName(), de.getDeletedProperties());
    }

    if (r && de.getIsUsed())
        notes->refreshCurrentItem();

    if (r)
        setModified(true);
}

void App::deleteEntry()
{
    ChooseEntry ce(this, 0, true);
    Entry *entry;

    ce.fillList(entriesList);
    if (!ce.exec())
        return;

    if (ce.currentEntry() == -1)
        return;

    entry = entriesList->at(ce.currentEntry());

    if (notes->isEntryUsed(entry->getName()))
    {
        QMessageBox::warning(this, tr("ٹ"), tr("EntryϥΡȤǻȤƤޤ\n뤳ȤǤޤ"), 0, 0, 0);

        return;
    }
    
    entriesList->remove
    (entry->getName());

    setModified(true);
}

void App::preferenc()
{
    PreferencesD pb(this, 0, true);

    pb.ShowReminder->setChecked(preferences.showReminder);
    pb.WordWrap->setChecked(config.wordWrap);
    pb.RootNodeDecoration->setChecked(config.rootNodeDecoration);
    pb.ShowScrollBars->setChecked(config.showScrollBars);
    pb.VerticalLayout->setChecked(config.verticalLayout);
    pb.verticalLayoutClicked();
    pb.InputDate->setChecked(config.inputDate);

    pb.SaveBackup->setChecked(preferences.saveBackupFile);
    pb.BackupLocation->setText(preferences.backupLocation);
    pb.HorPos->setValue(config.horPos);
    pb.VerPos->setValue(config.verPos);
    pb.AutoSave->setChecked(config.autoSave);
    pb.AutoLoad->setChecked(config.autoLoad);
    pb.AutoLoadFile->setText(config.autoLoadFile);
    
    if(config.dataLocation == "")
    	config.dataLocation = Global::applicationFileName("iqnotes","");
    pb.DataLocation->setText(config.dataLocation);

    // passwd
    if (!preferences.passwdHex.length())
    {
        pb.CurrentPasswd->setEnabled(false);
        pb.CurrentPasswdLabel->setEnabled(false);
        pb.DontEncrypt->setEnabled(false);
    }

    pb.setCurrentPasswd(preferences.passwdHex);
    pb.showMaximized();
    if (!pb.exec())
        return;

    if (pb.DontEncrypt->isChecked())
    {
        preferences.setPasswd("");
    }
    else if (pb.Passwd1->text().length())
    {
        preferences.setPasswd(pb.Passwd1->text());
    }

    preferences.showReminder = pb.ShowReminder->isChecked();
    config.wordWrap = pb.WordWrap->isChecked();
    config.rootNodeDecoration = pb.RootNodeDecoration->isChecked();
    config.showScrollBars = pb.ShowScrollBars->isChecked();
    config.verticalLayout = pb.VerticalLayout->isChecked();
    config.inputDate = pb.InputDate->isChecked();
    config.horPos = pb.HorPos->value();
    config.verPos = pb.VerPos->value();
    config.autoSave = pb.AutoSave->isChecked();
    config.autoLoad = pb.AutoLoad->isChecked();
    config.autoLoadFile = pb.AutoLoadFile->text();
    config.dataLocation = pb.DataLocation->text();

    // font changed
    if (config.font != pb.Font->currentText() || config.fontSize != pb.FontSize->currentText().toInt())
    {
        config.font = pb.Font->currentText();
        config.fontSize = pb.FontSize->currentText().toInt();
	    changeFont();

    }

    preferences.saveBackupFile = pb.SaveBackup->isChecked();
    preferences.backupLocation = pb.BackupLocation->text();
    notes->setWordWrap(config.wordWrap);
    notes->setRootNodeDecoration(config.rootNodeDecoration);
    notes->setShowScrollBars(config.showScrollBars);
    notes->setVerticalLayout(config.verticalLayout);
    notes->setInputDate(config.inputDate);
    notes->setVerPos(config.verPos);
    notes->setHorPos(config.horPos);

	Notes::EViewType viewType = notes->getViewType();
    
    if (viewType == Notes::HALF_VIEW){
        hideTree();
        halfView();
    }

    saveConfig();

    setModified(true);
}

// }}}

// {{{ Help menu

void App::about()
{
    AboutForm aboutD(this, tr("About Dialog"), true);

    // aboutD.showMaximized();
    aboutD.adjustSize();
    aboutD.exec();
}

// }}}

// }}}

void App::exportText()
{
	exportFile(1);
}

void App::exportSketch()
{
	exportFile(2);
}

void App::exportFile(int type)
{
	QDateTime dt = QDateTime::currentDateTime();
	QString tmp = "";
	tmp.sprintf("%04d%02d%02d_%02d%02d", dt.date().year(),dt.date().month(),dt.date().day(),dt.time().hour(),dt.time().minute());

	if(config.dataLocation == "")
		notes->exportFile(type, Global::applicationFileName("iqnotes", tmp));
	else
		notes->exportFile(type, config.dataLocation + tmp);

}

void App::execCommand()
{
	notes->execCommand();
}

void App::removeFile()
{
    OpenFile of(2,config.dataLocation,false, this, 0, true);
    connect(&of, SIGNAL(fileSelected(int, const QString &)), this, SLOT(removeFileA(int,const QString &)));

    of.showMaximized();

    if (!of.exec())
       return;

}
void App::removeFileA(int index,const QString &file)
{

	index = 1;
    int r = QMessageBox::warning(this, tr("γǧ"), tr("ե[" + file + "]\nޤ" ), QMessageBox::Yes | QMessageBox::Default, QMessageBox::No | QMessageBox::Escape);

    if (r != QMessageBox::Yes)
    	return;

	QString tmp = "";
	if(config.dataLocation == "")
		tmp = Global::applicationFileName("iqnotes", "");
	else
		tmp = config.dataLocation;

    QFile xmlFile(tmp + file + ".xml"),xmlRijndaelFile(tmp + file + ".rijn");
	xmlFile.remove();
	xmlRijndaelFile.remove();
}

void App::openFileWZ()
{

    if (!closeFile())
        return;

    OpenFile of(3,config.dataLocation,false, this, 0, true);
    connect(&of, SIGNAL(fileSelected(int, const QString &)), this, SLOT(openTreeWZ(int, const QString &)));

    of.showMaximized();

    if (!of.exec())
       return;

}

void App::openFileText()
{

    OpenFile of(3,config.dataLocation,false, this, 0, true);
    connect(&of, SIGNAL(fileSelected(int, const QString &)), this, SLOT(openTreeText(int, const QString &)));

    of.showMaximized();

    if (!of.exec())
       return;

}

void App::openTreeWZ(int index, const QString &file)
{

    // no file selected
    if (index == -1)
        return;

    createNotes();
    if (!openTreeInport(2,file))
    {
        destroyNotes();
        return;
    }
    
    currentFile = file;
    setModified(true);
    
    foreignDeletedNode = !notes->isClipboardEmpty();
    
    if (preferences.showReminder)
        notes->showReminder(false);
}

void App::openTreeText(int index, const QString &file)
{

    // no file selected
    if (index == -1)
        return;

    if (!openTreeInport(1,file))
    {
        destroyNotes();
        return;
    }
    
    currentFile = file;
    setModified(true);
    
    foreignDeletedNode = !notes->isClipboardEmpty();
    
    if (preferences.showReminder)
        notes->showReminder(false);
}

bool App::openTreeInport(int mode,const QString &file)
{
	QString tmp = "";
	if(config.dataLocation == "")
		tmp = Global::applicationFileName("iqnotes", "");
	else
		tmp = config.dataLocation;

    QFile txtFile(tmp + file + ".txt");
    if ( !txtFile.open( IO_ReadOnly ) )
        return false;
    
     QTextCodec *codec=QTextCodec::codecForName("ShiftJIS");

  	QTextStream ts( &txtFile );
  	ts.setCodec( codec );

	if(mode == 1)
		notes->readTreeText(&ts);
	else
		notes->readTreeWZ(&ts);

  	txtFile.close();

    setCaption("IQNotes :: " + file);
     
    return true;
}